/*
 * Copyright (C) 2001, 2002 The Mir-coders group
 *
 * This file is part of Mir.
 *
 * Mir is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Mir is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Mir; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * In addition, as a special exception, The Mir-coders gives permission to link
 * the code of this program with  any library licensed under the Apache Software License,
 * The Sun (tm) Java Advanced Imaging library (JAI), The Sun JIMI library
 * (or with modified versions of the above that use the same license as the above),
 * and distribute linked combinations including the two.  You must obey the
 * GNU General Public License in all respects for all of the code used other than
 * the above mentioned libraries.  If you modify this file, you may extend this
 * exception to your version of the file, but you are not obligated to do so.
 * If you do not wish to do so, delete this exception statement from your version.
 */

import java.io.IOException;
import java.io.PrintWriter;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import mir.servlet.AbstractServlet;
import mir.servlet.ServletModuleUserExc;
import mir.util.ExceptionRoutines;
import mircoders.servlet.ServletModuleOpenIndy;

/**
 *  OpenMir.java - main servlet for open posting and comment feature to articles
 *
 *  @author RK 1999-2001, the mir-coders group
 *  @version $Id: OpenMir.java,v 1.35.2.7 2005/10/01 18:40:40 zapata Exp $
 *
 */


public class OpenMir extends AbstractServlet {
  private ServletModuleOpenIndy servletModule;

  private synchronized ServletModuleOpenIndy getServletModule() {
    if (servletModule == null) {
      servletModule = new ServletModuleOpenIndy();
    }

    return servletModule;
  }

  public void process(HttpServletRequest aRequest,
                      HttpServletResponse aResponse) throws ServletException, IOException {
    if ("".equals(configuration.getString("RootUri", ""))) {
      configuration.setProperty("RootUri", aRequest.getContextPath());
    }

    long startTime = System.currentTimeMillis();

    HttpSession session = aRequest.getSession();

    checkLanguage(session, aRequest);

    //make sure client browsers don't cache anything
    setNoCaching(aResponse);

    aResponse.setContentType("text/html; charset="+configuration.getString("Mir.DefaultHTMLCharset"));

    try {
      // call appropriate ServletModuleOpenIndy method 
      // as specified in http request 
      // example: OpenMir?do=opensession calls the opensession method

      getServletModule().handleRequest(aRequest, aResponse);
    }
    catch (Throwable e) {
      logger.warn("Error in open publishing", e);

      Throwable cause = ExceptionRoutines.traceCauseException(e);

      if (cause instanceof ServletModuleUserExc) {
        handleUserError(aRequest, aResponse, aResponse.getWriter(), (ServletModuleUserExc) cause);
      }
      else {
        handleError(aRequest, aResponse, aResponse.getWriter(), cause);
      }
    }

    long sessionConnectTime = System.currentTimeMillis() - startTime;

    logger.debug("EXECTIME (ServletModuleOpenIndy): " + sessionConnectTime + " ms");
  }

  private void handleUserError(HttpServletRequest aRequest, HttpServletResponse aResponse,
                               PrintWriter anOutputWriter, ServletModuleUserExc anException) {
    getServletModule().handleUserError(aRequest, aResponse, anOutputWriter, anException);
  }

  private void handleError(HttpServletRequest aRequest, HttpServletResponse aResponse,PrintWriter out, Throwable anException) {
    getServletModule().handleError(aRequest, aResponse, out, anException);
  }

  public String getServletInfo(){
    return "OpenMir "+configuration.getString("Mir.Version");
  }
}

