/*
 * Copyright (C) 2005 The Mir-coders group
 *
 * This file is part of Mir.
 *
 * Mir is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Mir is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Mir; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * In addition, as a special exception, The Mir-coders gives permission to link
 * the code of this program with  any library licensed under the Apache Software License.
 * You must obey the GNU General Public License in all respects for all of the code used
 * other than the above mentioned libraries.  If you modify this file, you may extend this
 * exception to your version of the file, but you are not obligated to do so.
 * If you do not wish to do so, delete this exception statement from your version.
 */
package mir.bundle;

import java.util.Iterator;
import java.util.List;
import java.util.Properties;

import mir.util.StringParseRoutines;

/**
 * Bundle implementation using a {@link Properties}
 * object.
 */

public class PropertiesBundle implements Bundle {
  private Properties properties;

  public PropertiesBundle(Properties aProperties) {
    properties = aProperties;
  }

  private String formatValue(String anExpression, List anArguments) {
    StringBuffer result = new StringBuffer();

    List parts = StringParseRoutines.parseBracketedExpression(
        anExpression, '\\', "{", "}" );

    Iterator i = parts.iterator();
    while (i.hasNext()) {
      result.append(i.next());
      if (i.hasNext()) {
        String expression = (String) i.next();
        try {
          int parameterNr = Integer.parseInt(expression);
          if (parameterNr>=0 && parameterNr<anArguments.size()) {
            result.append(anArguments.get(parameterNr));
          }
        }
        catch (Throwable t) {
        }
      }
    }

    return result.toString();
  }

  /** {@inheritDoc} */
  public String getValue(String aKey, List anArguments) {
    String result = (String) properties.get(aKey);

    if (result==null || result.trim().length()==0)
      return null;
		return formatValue(result, anArguments);
  }
}
