/*
 * Copyright (C) 2001, 2002 The Mir-coders group
 *
 * This file is part of Mir.
 *
 * Mir is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Mir is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Mir; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * In addition, as a special exception, The Mir-coders gives permission to link
 * the code of this program with  any library licensed under the Apache Software License,
 * The Sun (tm) Java Advanced Imaging library (JAI), The Sun JIMI library
 * (or with modified versions of the above that use the same license as the above),
 * and distribute linked combinations including the two.  You must obey the
 * GNU General Public License in all respects for all of the code used other than
 * the above mentioned libraries.  If you modify this file, you may extend this
 * exception to your version of the file, but you are not obligated to do so.
 * If you do not wish to do so, delete this exception statement from your version.
 */
package  mir.module;

import mir.entity.Entity;
import mir.entity.EntityList;
import mir.storage.Database;

import java.util.Map;

/**
 * This class provides the base functionality for the derived Module-Classes.
 * These classes should provide methods to make more or less complex actions
 * on Database and Entity classes. The modules are used by ServletModules.
 * Future possibility could be access via Applications.
 *
 * Abstrakte Klasse, von denen die Modules die Basisfunktionalit?t erben.
 * Die Moduleschicht dient dazu, Funktionalitaeten zur Verf?gung zu stellen,
 * die von mehreren ServletModulen verwendet werden.
 *
 */

public class AbstractModule {
  protected Database database;

  public AbstractModule(Database aDatabase) {
    database = aDatabase;
  }

  /**
   * Returns the database object associated with this module
   */
  public Database getStorageObject () {
    return database;
  }

  public Entity getById (String id) throws ModuleExc, ModuleFailure {
    try {
      if (database == null)
        throw  new ModuleExc("AbstractModule.getById: No Database set!");
      Entity entity = database.selectById(id);

      if (entity == null)
        throw new ModuleExc("AbstractModule.getById: No object for id = " + id);
			return entity;
    }
    catch (Throwable e) {
      throw new ModuleFailure(e);
    }
  }

  public EntityList getByWhereClause (String whereClause, int offset) throws ModuleExc, ModuleFailure {
    try {
      if (database == null)
        throw  new ModuleExc("AbstractModule.getByWhereClause: No Database set!");

      return database.selectByWhereClause(whereClause, offset);
    }
    catch (Throwable e) {
      throw new ModuleFailure(e);
    }
  }

  public String add(Map theValues) throws ModuleExc, ModuleFailure {
    try {
      Entity entity = database.createNewEntity();
      entity.setFieldValues(theValues);

      return entity.insert();
    }
    catch (Throwable e) {
      throw new ModuleFailure(e);
    }
  }

  public String set (Map theValues) throws ModuleExc, ModuleFailure {
    try {
      Entity theEntity = database.selectById((String) theValues.get("id"));
      if (theEntity == null)
        throw new ModuleExc("No object found with id " + theValues.get("id"));
      theEntity.setFieldValues(theValues);
      theEntity.update();
      return theEntity.getId();
    }
    catch (Throwable e) {
      throw new ModuleFailure(e);
    }
  }

  /**
   * Deletes a record using an id
   * @param idParam
   * @exception ModuleExc
   * @exception ModuleFailure
   */
  public void deleteById (String idParam) throws ModuleExc, ModuleFailure {
    try {
      database.delete(idParam);
    }
    catch (Throwable e) {
      throw new ModuleFailure(e);
    }
  }
}
