/*
 * Copyright (C) 2001, 2002 The Mir-coders group
 *
 * This file is part of Mir.
 *
 * Mir is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Mir is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Mir; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * In addition, as a special exception, The Mir-coders gives permission to link
 * the code of this program with  any library licensed under the Apache Software License, 
 * The Sun (tm) Java Advanced Imaging library (JAI), The Sun JIMI library 
 * (or with modified versions of the above that use the same license as the above), 
 * and distribute linked combinations including the two.  You must obey the 
 * GNU General Public License in all respects for all of the code used other than 
 * the above mentioned libraries.  If you modify this file, you may extend this 
 * exception to your version of the file, but you are not obligated to do so.  
 * If you do not wish to do so, delete this exception statement from your version.
 */
package mir.session;

import mir.util.IORoutines;
import org.apache.commons.fileupload.FileItem;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

public class CommonsUploadedFileAdapter implements UploadedFile {
  private FileItem fileItem;

  public CommonsUploadedFileAdapter(FileItem aFileItem) {
    fileItem = aFileItem;
  }

  public void writeToFile(File aFile) throws SessionExc, SessionFailure {
    try {
      aFile.getParentFile().mkdirs();
      fileItem.write(aFile);
    }
    catch (Exception e) {
      throw new SessionFailure(e);
    }
  }

  public void writeToStream(OutputStream aStream) throws SessionExc, SessionFailure {
    try {
      IORoutines.copyStream(fileItem.getInputStream(), aStream);
    }
    catch (Exception e) {
      throw new SessionFailure(e);
    }
  }

  public InputStream getInputStream() throws SessionExc, SessionFailure {
    try {
      return fileItem.getInputStream();
    }
    catch (IOException e) {
      throw new SessionFailure(e);
    }
  }

  public String getFileName() {
    return fileItem.getName();
  }

  public String getFieldName() {
    return fileItem.getFieldName();
  }

  public String getContentType() {
    return fileItem.getContentType();
  }
}