/*
 * Copyright (C) 2001, 2002 The Mir-coders group
 *
 * This file is part of Mir.
 *
 * Mir is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Mir is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Mir; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * In addition, as a special exception, The Mir-coders gives permission to link
 * the code of this program with  any library licensed under the Apache Software License, 
 * The Sun (tm) Java Advanced Imaging library (JAI), The Sun JIMI library 
 * (or with modified versions of the above that use the same license as the above), 
 * and distribute linked combinations including the two.  You must obey the 
 * GNU General Public License in all respects for all of the code used other than 
 * the above mentioned libraries.  If you modify this file, you may extend this 
 * exception to your version of the file, but you are not obligated to do so.  
 * If you do not wish to do so, delete this exception statement from your version.
 */
package mir.util;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.lang.reflect.InvocationTargetException;

import multex.Failure;

import org.xml.sax.SAXException;

/**
 * Routines to assist in handling and throwing exceptions</p>
 */

public class ExceptionRoutines {
  protected ExceptionRoutines() {
  }

  /**
   * Traces an exception to it's root cause, using all known exception types that support
   *   cause exceptions.
   *
   * @return the cause (if found)
   */

  public static Throwable traceCauseException(Throwable anException) {
    Throwable result = anException;

    while (true) {
      if ((result instanceof Failure) && (((Failure) result).getCause()!=null)) {
        result = ((Failure) result).getCause();
      }
      else if ((result instanceof SAXException) && (((SAXException) result).getException()!=null)) {
        result = ((SAXException) result).getException();
      }
      else if ((result instanceof InvocationTargetException) && (((InvocationTargetException) result).getTargetException()!=null)) {
        result = ((InvocationTargetException) result).getTargetException();
      }
      else
        break;
    }

    return result;
  }

  /**
   * Prints an exception's stacktrace to a <code>String</code> 
   */
  public static String getStackTrace(Throwable aThrowable) {
    StringWriter writer = new StringWriter();
    aThrowable.printStackTrace(new PrintWriter(writer));
    return writer.toString();
  }
}
