/*
 * Copyright (C) 2001, 2002 The Mir-coders group
 *
 * This file is part of Mir.
 *
 * Mir is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Mir is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Mir; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * In addition, as a special exception, The Mir-coders gives permission to link
 * the code of this program with  any library licensed under the Apache Software License,
 * The Sun (tm) Java Advanced Imaging library (JAI), The Sun JIMI library
 * (or with modified versions of the above that use the same license as the above),
 * and distribute linked combinations including the two.  You must obey the
 * GNU General Public License in all respects for all of the code used other than
 * the above mentioned libraries.  If you modify this file, you may extend this
 * exception to your version of the file, but you are not obligated to do so.
 * If you do not wish to do so, delete this exception statement from your version.
 */

package mircoders.entity;

import mir.media.image.ImageMagickImageProcessor;
import mir.media.image.ImageProcessor;
import mir.media.MediaExc;
import mir.storage.DatabaseFailure;

import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.io.IOException;
import java.sql.SQLException;

public class EntityImages extends EntityUploadedMedia
{
  private int maxImageSize = configuration.getInt("Producer.Image.MaxSize");
  private int maxIconSize = configuration.getInt("Producer.Image.MaxIconSize");
  private float minDescaleRatio = configuration.getFloat("Producer.Image.MinDescalePercentage")/100;
  private int minDescaleReduction = configuration.getInt("Producer.Image.MinDescaleReduction");


  public EntityImages()
  {
    super();
  }

  /**
   * Retrieves the image data
   */
  public byte[] getImage() throws DatabaseFailure {
    try {
      return database.getBinaryField("select image_data from images where id="+getId());
    }
    catch (SQLException e) {
      throw new DatabaseFailure(e);
    }
  }

  /**
   * Processes and saves image data
   */
  public void setImage(InputStream anInputStream, String type) throws IOException, MediaExc, SQLException {
    if (anInputStream != null) {
        ImageProcessor processor = new ImageMagickImageProcessor(anInputStream);

        processor.descaleImage(maxImageSize, minDescaleRatio, minDescaleReduction);

        ByteArrayOutputStream imageData = new ByteArrayOutputStream();
        processor.writeScaledData(imageData, type);
        database.setBinaryField("image_data", getId(), imageData.toByteArray());

        setFieldValue("img_height", Integer.toString(processor.getScaledHeight()));
        setFieldValue("img_width", Integer.toString(processor.getScaledWidth()));

        imageData.reset();
        processor.descaleImage(maxIconSize, minDescaleRatio, minDescaleReduction);
        processor.writeScaledData(imageData, type);
        database.setBinaryField("icon_data", getId(), imageData.toByteArray());

        setFieldValue("icon_height", Integer.toString(processor.getScaledHeight()));
        setFieldValue("icon_width", Integer.toString(processor.getScaledWidth()));
        processor.cleanup();
        update();
    }
  }

  /**
   * Takes an OutputStream as an argument and reads in the data
   * from the DB and writes it to the OutputStream.
   *
   * It will also take care of closing the OutputStream.
   */
  public byte[] getIcon() throws DatabaseFailure {
    try {
      return database.getBinaryField("select icon_data from images where id="+getId());
    }
    catch (SQLException e) {
      throw new DatabaseFailure(e);
    }
  }
}
